@@ -9,7 +9,7 @@ use base qw/
 use Carp::Clan qw/^DBIx::Class/;
 use Class::C3;
 
-our $VERSION = '0.05003';
+our $VERSION = '0.06000';
 
 =head1 NAME
 
@@ -73,7 +73,7 @@ sub _table_uniq_info {
 
 # DBD::DB2 doesn't follow the DBI API for ->tables
 sub _tables_list { 
-    my $self = shift;
+    my ($self, $opts) = @_;
     
     my $dbh = $self->schema->storage->dbh;
     my @tables = map { lc } $dbh->tables(
@@ -82,7 +82,7 @@ sub _tables_list {
     s/\Q$self->{_quoter}\E//g for @tables;
     s/^.*\Q$self->{_namesep}\E// for @tables;
 
-    return @tables;
+    return $self->_filter_tables(\@tables, $opts);
 }
 
 sub _table_pk_info {
@@ -108,11 +108,9 @@ sub _columns_info_for {
 }
 
 sub _extra_column_info {
-    my ($self, $info) = @_;
+    my ($self, $table, $column, $info, $dbi_info) = @_;
     my %extra_info;
 
-    my ($table, $column) = @$info{qw/TABLE_NAME COLUMN_NAME/};
-
     my $dbh = $self->schema->storage->dbh;
     my $sth = $dbh->prepare_cached(
         q{
