@@ -6,16 +6,16 @@ use base 'DBIx::Class::Schema::Loader::DBI';
 use Carp::Clan qw/^DBIx::Class/;
 use Class::C3;
 
-our $VERSION = '0.05003';
+our $VERSION = '0.06000';
 
 =head1 NAME
 
-DBIx::Class::Schema::Loader::DBI::Sybase::Common - Common functions for Sybase
+DBIx::Class::Schema::Loader::DBI::Sybase::Common - Common methods for Sybase
 and MSSQL
 
 =head1 DESCRIPTION
 
-See L<DBIx::Class::Schema::Loader::Base>.
+See L<DBIx::Class::Schema::Loader> and L<DBIx::Class::Schema::Loader::Base>.
 
 =cut
 
@@ -23,14 +23,14 @@ See L<DBIx::Class::Schema::Loader::Base>.
 sub _build_quoter  { '"' }
 sub _build_namesep { '.' }
 
-sub _set_quote_char_and_name_sep {
+sub _setup {
     my $self = shift;
 
-    $self->schema->storage->sql_maker->quote_char([qw/[ ]/])
-        unless $self->schema->storage->sql_maker->quote_char;
+    $self->next::method(@_);
 
-    $self->schema->storage->sql_maker->name_sep('.')
-        unless $self->schema->storage->sql_maker->name_sep;
+    $self->schema->storage->sql_maker->quote_char([qw/[ ]/]);
+    $self->schema->storage->sql_maker->name_sep('.');
+    $self->{db_schema} ||= $self->_build_db_schema;
 }
 
 sub _build_db_schema {
@@ -80,6 +80,8 @@ sub _columns_info_for {
 
 L<DBIx::Class::Schema::Loader::DBI::Sybase>,
 L<DBIx::Class::Schema::Loader::DBI::MSSQL>,
+L<DBIx::Class::Schema::Loader::DBI::ODBC::Microsoft_SQL_Server>,
+L<DBIx::Class::Schema::Loader::DBI::Sybase::Microsoft_SQL_Server>,
 L<DBIx::Class::Schema::Loader::DBI>
 L<DBIx::Class::Schema::Loader>, L<DBIx::Class::Schema::Loader::Base>,
 
