@@ -6,37 +6,24 @@ use base 'DBIx::Class::Schema::Loader::DBI::MSSQL';
 use Carp::Clan qw/^DBIx::Class/;
 use Class::C3;
 
-our $VERSION = '0.05003';
+our $VERSION = '0.06000';
 
 =head1 NAME
 
-DBIx::Class::Schema::Loader::DBI::Sybase::Microsoft_SQL_Server - Subclass for
-using MSSQL through DBD::Sybase
+DBIx::Class::Schema::Loader::DBI::Sybase::Microsoft_SQL_Server - Driver for
+using Microsoft SQL Server through DBD::Sybase
 
 =head1 DESCRIPTION
 
-See L<DBIx::Class::Schema::Loader::Base>.
-
 Subclasses L<DBIx::Class::Schema::Loader::DBI::MSSQL>.
 
-=cut
-
-# Returns an array of table names
-sub _tables_list { 
-    my $self = shift;
-
-    my ($table, $type) = @_ ? @_ : ('%', '%');
-
-    my $dbh = $self->schema->storage->dbh;
-    my @tables = $dbh->tables(undef, $self->db_schema, $table, $type);
-
-    return $self->_filter_tables(@tables);
-}
+See L<DBIx::Class::Schema::Loader> and L<DBIx::Class::Schema::Loader::Base>.
 
 =head1 SEE ALSO
 
-L<DBIx::Class::Schema::Loader::DBI::Sybase>,
 L<DBIx::Class::Schema::Loader::DBI::MSSQL>,
+L<DBIx::Class::Schema::Loader::DBI::ODBC::Microsoft_SQL_Server>,
+L<DBIx::Class::Schema::Loader::DBI::Sybase::Common>,
 L<DBIx::Class::Schema::Loader::DBI>
 L<DBIx::Class::Schema::Loader>, L<DBIx::Class::Schema::Loader::Base>,
 
